// Timer driver for nRF52840 | gat 11/04/23
#include "Timer.h"
#include "nrf52840.h"
#include "nrf52840_bitfields.h"

static void dummy(void){}

static void (*Callbacks[5][5])(void) = {{dummy, dummy, dummy, dummy, dummy},
{dummy, dummy, dummy, dummy, dummy},{dummy, dummy, dummy, dummy, dummy},
{dummy, dummy, dummy, dummy, dummy},{dummy, dummy, dummy, dummy, dummy}};

static NRF_TIMER_Type* GetReg(Timer_Channel ch)
{
	switch(ch)
	{
		default:
		case Timer0:
		return NRF_TIMER0;
		case Timer1:
		return NRF_TIMER1;
		case Timer2:
		return NRF_TIMER2;
		case Timer3:
		return NRF_TIMER3;
		case Timer4:
		return NRF_TIMER4;
	}
}

void Timer_Setup(Timer_Config cfg)
{
	NRF_TIMER_Type* reg = GetReg(cfg.Channel);
	reg->TASKS_STOP = 1U;
	reg->PRESCALER = cfg.Prescaler;
	reg->BITMODE = cfg.Bitmode;
}

void Timer_Set_Compare(Timer_Compare_Config cfg)
{
	NRF_TIMER_Type* reg = GetReg(cfg.Channel);
	reg->TASKS_STOP = 1U;
	reg->CC[(unsigned char)cfg.Compare_Channel] = cfg.CompareValue;
	reg->INTENSET = 1U << (cfg.Compare_Channel + 16U);
	switch (cfg.Shortcut)
	{
		default:
		case None:
			break;
		case Clear:
			reg->SHORTS = 1U << cfg.Compare_Channel;
			break;
		case Stop:
			reg->SHORTS = 1U << (cfg.Compare_Channel + 8U);
			break;
	}
	Callbacks[cfg.Channel][cfg.Compare_Channel] = cfg.Callback;
}

void Timer_Start(Timer_Channel ch)
{
	NRF_TIMER_Type* reg = GetReg(ch);
	reg->TASKS_START = 1U;
}

void Counter_Setup(Counter_Config cfg)
{
	NRF_TIMER_Type* reg = GetReg(cfg.Channel);
	unsigned int cc_ch = (unsigned int)cfg.Compare_Channel;
	reg->TASKS_STOP = 1U;
	reg->TASKS_CLEAR = 1U;
	reg->MODE = TIMER_MODE_MODE_Counter;
	reg->BITMODE = cfg.Bitmode;
	reg->SHORTS = 1U << cc_ch; // Compare[i] -> Clear
	reg->CC[cc_ch] = cfg.CompareValue;
	reg->INTENSET = 1U << (cc_ch + 16U); // Compare[i] -> IRQ
	reg->TASKS_START = 1U;
	Callbacks[cfg.Channel][cfg.Compare_Channel] = cfg.Callback;
}

void Counter_Clear(Timer_Channel chann)
{ 
	NRF_TIMER_Type* reg = GetReg(chann);
	reg->TASKS_STOP = 1U;
	reg->TASKS_CLEAR = 1U;
	reg->TASKS_START = 1U;
}

void TIMER0_IRQHandler(void)
{
	for(unsigned int i = 0; i < 5; i++)
	if(NRF_TIMER0->EVENTS_COMPARE[i])
	{
		NRF_TIMER0->EVENTS_COMPARE[i] = 0;
		Callbacks[0][i]();
	}
}

void TIMER1_IRQHandler(void)
{
	for(unsigned int i = 0; i < 5; i++)
	if(NRF_TIMER1->EVENTS_COMPARE[i])
	{
		NRF_TIMER1->EVENTS_COMPARE[i] = 0;
		Callbacks[1][i]();
	}
}

void TIMER2_IRQHandler(void)
{
	for(unsigned int i = 0; i < 5; i++)
	if(NRF_TIMER2->EVENTS_COMPARE[i])
	{
		NRF_TIMER2->EVENTS_COMPARE[i] = 0;
		Callbacks[2][i]();
	}
}

void TIMER3_IRQHandler(void)
{
	for(unsigned int i = 0; i < 5; i++)
	if(NRF_TIMER3->EVENTS_COMPARE[i])
	{
		NRF_TIMER3->EVENTS_COMPARE[i] = 0;
		Callbacks[3][i]();
	}
}

void TIMER4_IRQHandler(void)
{
	for(unsigned int i = 0; i < 5; i++)
	if(NRF_TIMER4->EVENTS_COMPARE[i])
	{
		NRF_TIMER4->EVENTS_COMPARE[i] = 0;
		Callbacks[4][i]();
	}
}
