#ifndef _IMPLANT_HPP_
#define _IMPLANT_HPP_

#include "Radio.h"

#define IMPLANT_BUF_SIZE 130

typedef enum
{
	Implant_Disabled = 0, 
	Implant_Standby , 
	Implant_Scan ,
	Implant_NoScan ,
	Implant_Listening ,
	Implant_NoListening ,
	Implant_NoListening_PAD
} Implant_Status_t;

typedef struct
{
	Radio_Config radio_config;
	unsigned char RF_period; // frame period (ms)
	unsigned char ACQ_period; // min acq period (ms)	
	bool QOS_enable;
} Implant_Config_t;

typedef struct 
{
	unsigned int NL; // Not Listening
	unsigned int NL_PAD; // Not Listening Padding
	unsigned int L; // Listening
	unsigned int PULSE; // Pulse
	unsigned int SCAN; // Scan
} Wait_Time_t;

typedef struct
{
	bool Enable;
	unsigned char Resolution;
	unsigned char Data[IMPLANT_BUF_SIZE]; // TODO check
	unsigned char DataWriteIndex;
	unsigned char DataReadIndex;
	unsigned char SamplesCountPerFrame;
	unsigned char SamplesCount;
	unsigned short LastSample;
} ImplantSignal;

typedef struct
{
	unsigned char Ts_Slow;
	ImplantSignal iFast1;
	ImplantSignal iFast2;
	ImplantSignal iFast3;
	ImplantSignal iFast4;
	ImplantSignal iT;
	ImplantSignal iAcc;
	ImplantSignal iVbat;
	ImplantSignal QoS;
} ImplantSignalConfig;

typedef enum
{
	iFast1,
	iFast2,
	iFast3,
	iFast4,
	iT,
	iAcc,
	iVbat,
	QoS
} ImplantSignalName;

class TImplant
{
	public :
		static void StartConfig(unsigned char* cfg_ptr);
		static void GetSample(unsigned char* data_ptr, ImplantSignalName signal_name);

	private :
		static void PopulateSignals(unsigned char* cfg_ptr);
		static void CalculateWaitTimes(void);
		static unsigned int CalculateFrameTime(void);
		static void RTC_Tick(void);
		static void Frame_Received(void);
		static void Add_QoS_Sample(bool data);
		static void ProcessFrame(void);
		static void ProcessSignal(ImplantSignal* signal);
		static unsigned short GetSigSample(ImplantSignal* signal);
		static unsigned short GetQoSSample(void);
		static void StartRTC(unsigned int time);

		static ImplantSignalConfig Configuration;
		static Implant_Status_t Status;
		static unsigned int nb_PAD; // lost frame count
		static Implant_Config_t Implant_Config;
		static Wait_Time_t Wait_Times;
		static unsigned short IndexRead;
		static unsigned char Implant_Data[IMPLANT_BUF_SIZE];
		static double datarate[3];
		static bool sample_timeout_flag;
};

#endif //_IMPLANT_HPP_
